/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// kickaction.cpp Fri Jun 15 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "kickaction.h"
#include "kickcommand.h"
#include "dashcommand.h"

bool KickAction::fTriedBackKick = false;

KickAction::KickAction() : Action()
{
	// A kickaction takes one 100ms timestep, so iDuration is set to 1.
	// in the future this value may changed based on the worldmodel
	// and the server commands (may need to be dynamic)
	setDuration(1);
}

KickAction::KickAction(const Angle &_aDirection, double _dIntensity) : Action()
{
	setDuration(1);
	setDirection(_aDirection);
	setIntensity(_dIntensity);
}


KickAction::~KickAction()
{
}

Action* KickAction::copy() const
{
	return new KickAction(*this);
}

ServerCommand* KickAction::translate() const
{
	// WARNING: creating new object, remember to delete!
	// kick the ball in the desired direction... kick it to the side, if a back kick won't work
	KickCommand* pkcCmd;
	
	if ((getDirection() > (2.0 * PI_3)) || (getDirection() < (-2.0 * PI_3)) && KickAction::fTriedBackKick)
	{
		Angle aDir;
		
		if (getDirection() < 0)
			aDir = (-2.0 * PI_3);
		else
			aDir = (2.0 * PI_3);
		
		pkcCmd = new KickCommand(5, aDir);
		KickAction::fTriedBackKick = false;
	}
	else if ((getDirection() > PI_2) || (getDirection() < -PI_2) && !KickAction::fTriedBackKick)
	{
		pkcCmd = new KickCommand(getIntensity(), getDirection());
		KickAction::fTriedBackKick = true;
	}
	else
	{
		pkcCmd = new KickCommand(getIntensity(), getDirection());
		KickAction::fTriedBackKick = false;
	}
	
	return pkcCmd;

}

void KickAction::setDirection(const Angle &_aDirection)
{
	aDirection = _aDirection;
}

void KickAction::setIntensity(double _dIntensity)
{
	dIntensity = _dIntensity;
}

Angle KickAction::getDirection() const
{
	return aDirection;
}

double KickAction::getIntensity() const
{
	return dIntensity;
}

ostream& KickAction::ostream_insert(ostream& _out) const
{
    _out << "KickAction";
    return _out;
}
ofstream& KickAction::ofstream_insert(ofstream& _fout) const
{
    // TODO: Implement XML

    return _fout;
}
