/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// moveaction.cpp Mon Jun 11 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "moveaction.h"
#include "movecommand.h"
#include "dashcommand.h"
#include "turncommand.h"

MoveAction::MoveAction(): Action()
{
	// A kickaction takes one 100ms timestep, so iDuration is set to 1.
	// in the future this value may changed based on the worldmodel
	// and the server commands (may need to be dynamic)
	setDuration(1);
}

MoveAction::MoveAction(double _dX, double _dY) : Action()
{
	setDuration(1);
	setPosition(_dX, _dY);
}

MoveAction::MoveAction(Vector2d _vPosition) : Action()
{
    setDuration(1);
    setPosition(_vPosition);
}

MoveAction::~MoveAction()
{
}

Action* MoveAction::copy() const
{
	return new MoveAction(*this);
}

ServerCommand* MoveAction::translate() const
{
	// WARNING: creating a new servercommand, must remember to delete!!!
	ServerCommand* pscCmd;
	
	// pass the vector to the movecommand
	// TODO: make movecommand take a vector or this class use a vector
	pscCmd = new MoveCommand(vPosition.getX(), vPosition.getY());

	return pscCmd;
}

void MoveAction::setPosition(double _dX, double _dY)
{
	vPosition.setX(_dX);
	vPosition.setY(_dY);
}

void MoveAction::setPosition(Vector2d _vPosition)
{
    vPosition = _vPosition;
}

Vector2d MoveAction::getPosition() const
{
	return vPosition;
}

ostream& MoveAction::ostream_insert(ostream& _out) const
{
    _out << "MoveAction" << endl;

    return _out;
}
ofstream& MoveAction::ofstream_insert(ofstream& _fout) const
{
    // TODO: Implement XML

    return _fout;
}
