/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// lowlevelcomm.cpp Thu Jun 14 2001
// Description: LowLevelComm encapsilates the sending of information
//							to sockets as well as handling of SIGIO signals
//////////////////////////////////////////////////////////////////////

#include "lowlevelcomm.h"
#include "constants.h"
#include "parser.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <sys/file.h>

#include <fcntl.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netdb.h>

#undef TRACE
#define TRACE(cmd)

sigset_t     LowLevelComm::alrmask;
sigset_t     LowLevelComm::iomask;
sigset_t     LowLevelComm::bothmask;
sig_atomic_t LowLevelComm::alarmFlag;
sig_atomic_t LowLevelComm::ioFlag;
int          LowLevelComm::iWorldTime;

LowLevelComm::LowLevelComm()
{
//	aboutMe = &_aboutMe;
  serverOk = false;

	sigemptyset(&iomask);
	sigaddset(&iomask, SIGIO);
	sigemptyset(&alrmask);
	sigaddset(&alrmask, SIGALRM);
	alarmFlag = 0;
	ioFlag    = 0;

	sd = -1;
	
//    debug_out.open("net.log");
}

LowLevelComm::~LowLevelComm()
{
    removeSigHandlers();
    if (sd >= 0)
        close(sd);

//    debug_out.close();

}

// static member methods for signal handling
void LowLevelComm::alarmHandler(int signo)
{

    if (signo)
    {} // prevent warning when compiling



   TRACE( "SIGALRM");
    alarmAction();
}

void LowLevelComm::ioHandler(int signo)
{

    if (signo)
    {} // prevent warning when compiling

        TRACE( "SIGIO" << endl);
        ioAction();
}

// getTimeExpression returns the local time ??
int LowLevelComm::getTimeExpression()
{
    const int MAX_SEC = 100;
    struct timeval now;
    gettimeofday (&now, NULL);
    long time_ms = (now.tv_sec%MAX_SEC)*1000 + now.tv_usec/1000;
    return time_ms % 10000;
}

// sendToServer(char *msg) sends string msg to the soccer server
comm_error LowLevelComm::sendToServer(char *msg)
{

  // Prints the string and sends it to the server via the UDP socket.
  INSPECT(LowLevelComm::iWorldTime << " systime: " << getTimeExpression() 
          << " snd: "<< msg);
  
  //    debug_out << LowLevelComm::iWorldTime << " systime: " << getTimeExpression()
  //          << " snd: "<< msg << endl;
  
  
  if (strncmp(msg, "(say", 4) != 0)
    { // avoid logging say commands
      TRACE("servertime_here" << "> " << msg);
    }
  
  
    int len = strlen(msg);
    int actual;
    do
    { // repeat sending if interrupted
        actual = sendto(sd, msg, len, 0, (struct sockaddr *)
                        &server_addr, sizeof(server_addr));
    }
    while (actual < 0 && errno == EINTR);

    if (actual != len)
        return COMM_ERROR;
    else
        return COMM_OK;
}


comm_error LowLevelComm::receiveFromServer(char *msg, long timeout_msec)
{
    // select() the socket to find out if data is there. Then keep receiving
    // until the whole string was read from the socket, and return it.

    fd_set                watchMe;
    timeval               timeout;
    int	                n;
#ifdef linux
   	unsigned int          fromlen;
#else
		int										fromlen;
#endif
    struct sockaddr_in    from_addr;
    bool                  repeatSelect;

    // create the file descriptor set to be watched by select()
    FD_ZERO (&watchMe);
    FD_SET (sd, &watchMe);
    timeout.tv_sec  = (1000*timeout_msec) / 1000000;
    timeout.tv_usec = (1000*timeout_msec) % 1000000;

    do
    {
        repeatSelect = false;
        switch (select (sd+1, &watchMe, NULL, NULL, &timeout))
        {

        case 1: // the socket has data
            fromlen = sizeof(from_addr) ;

            do
            { // repeat receiving if interrupted
                n = recvfrom(sd, msg, MAX_SERVER_MSG_SIZE, 0,
                             (struct sockaddr *)&from_addr, &fromlen);
            }
            while ((n == -1) && (errno == EINTR));

            if (n<0)
                return COMM_ERROR; // not an EINTR error !
            msg[n] = 0;

            TRACE("servertime_here" << " systime: " << getTimeExpression()
                  << " rcv: " << msg << endl);

//            debug_out << LowLevelComm::iWorldTime << " systime: " << getTimeExpression()
//                  << " rcv: "<< msg << endl;
            // sender of this message will be next adressee (when receiving init)
            server_addr.sin_port = from_addr.sin_port;

            return COMM_OK;

        case 0: // socket had no data and timed out
            return COMM_ERROR_NO_DATA;

        case -1: // select() returned an error
            if (errno != EINTR)
                return COMM_ERROR; // select returned a serious error
            else
                repeatSelect = true; // repeat on interrupt
        } //switch
    } //do

    while (repeatSelect);

    // if control would reach this point, something were seriously wrong
    return COMM_ERROR;

}

// when alarmAction is called by alarmHandler switch alarmFlag to true
// see Comm::handleSignals for more information about the use of alarmFlag
void LowLevelComm::alarmAction()
{
	LowLevelComm::alarmFlag = 1;
}

// when ioAction is called by ioHandler switch ioFlag to true
// see Comm::handleSignals for more information about the use of ioFlag
void LowLevelComm::ioAction()
{
	LowLevelComm::ioFlag = 1;
}

comm_error LowLevelComm::createSocket (const char *hostname, const int port)
{
    // lookup the server's address, create a socket and bind it

    TRACE("LowLevelComm:     creating socket at "
          << hostname << ":" << port);

    // Init remote and local addresses
    memset(&server_addr, 0, sizeof(sockaddr_in));
    server_addr.sin_family = AF_INET;

    struct sockaddr_in local_addr;
    memset(&local_addr, 0, sizeof(sockaddr_in));
    local_addr.sin_family = AF_INET ;

    TRACE("LowLevelComm:         looking network address for "
          << hostname);
    // Look up the server's network address
    struct hostent *hp;
    if ((hp = gethostbyname(hostname)) != NULL)
        memcpy(&server_addr.sin_addr, hp->h_addr, sizeof(server_addr.sin_addr));
    else
        if ((server_addr.sin_addr.s_addr = inet_addr(hostname)) ==
                (unsigned long)-1)
            return COMM_ERROR_SERVER_NOT_FOUND;

    server_addr.sin_port = htons(port);


    TRACE("LowLevelComm:         creating socket on remote port "
          << port);
    // Create UDP socket
    if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        return COMM_ERROR_SOCKET_INIT;

    // get SIGIO delivered to current process
    if (fcntl (sd, F_SETOWN, getpid()))
        return COMM_ERROR_SOCKET_INIT;

    // make socket non-blocking
    int val = fcntl(sd, F_GETFL, 0) ;
    //val |= O_ASYNC; // BSD-style option, does not work properly !
    val |= FASYNC;
    // from CMU98: undocumented option, but the only one that works ;-)
    if (fcntl(sd, F_SETFL, val) )
        return COMM_ERROR_SOCKET_INIT;

    TRACE("LowLevelComm:         binding local address to socket");
    // Bind local address
    local_addr.sin_addr.s_addr = htonl(INADDR_ANY) ;
    local_addr.sin_port = htons(0) ;
    if (bind(sd, (struct sockaddr *)&local_addr, sizeof(local_addr)) < 0)
    {
        return COMM_ERROR_SOCKET_INIT;
    }
    TRACE("LowLevelComm:     socket sucessfully opened");

    return COMM_OK;
}

comm_error LowLevelComm::initSigHandlers()
{
    TRACE("LowLevelComm:     initializing signal handlers");

    struct sigaction saction;

    saction.sa_mask = alrmask; // SIGIO-exec blocks SIGALRM
    saction.sa_flags = SA_RESTART;
    saction.sa_handler = LowLevelComm::ioHandler; //(void (*) (...)) Comm::ioHandler;
    if (sigaction (SIGIO, &saction, NULL))
        return COMM_ERROR_SIGNAL_INIT;

    saction.sa_mask = iomask; // SIGALRM-exec blocks SIGIO
    saction.sa_flags = SA_RESTART;
    saction.sa_handler = LowLevelComm::alarmHandler; //(void (*) (...)) Comm::alarmHandler;
    if (sigaction (SIGALRM, &saction, NULL))
        return COMM_ERROR_SIGNAL_INIT;

    return COMM_OK;
}

comm_error LowLevelComm::setAlarmInterval(int msec)
{

    TRACE("LowLevelComm:     setting alarm interval to " <<msec<< " msec");

    long usec = msec*1000;

    itimerval itv;

    // time left in current interval
    itv.it_value.tv_sec     = usec / 1000000;
    itv.it_value.tv_usec    = usec % 1000000;

    // stadard interval time
    itv.it_interval.tv_sec  = usec / 1000000;
    itv.it_interval.tv_usec = usec % 1000000;

    if (setitimer (ITIMER_REAL, &itv, NULL ))
        return COMM_ERROR_SIGNAL_INIT;
    else
        return COMM_OK;

}

void LowLevelComm::removeSigHandlers()
{

    TRACE("LowLevelComm:     removing all signal handlers");

    struct sigaction saction;

    sigset_t emptymask;
    sigemptyset(&emptymask);

    saction.sa_mask = emptymask;
    saction.sa_flags = SA_RESTART;
    saction.sa_handler = SIG_DFL;

    sigaction(SIGIO, &saction, NULL);
    sigaction(SIGALRM, &saction, NULL);

}

comm_error LowLevelComm::blockSignals()
{

    TRACE("LowLevelComm:     blocking all signals");

    if ( sigprocmask(SIG_BLOCK, &alrmask, NULL) ||
            sigprocmask(SIG_BLOCK, &iomask, NULL)  )
        return COMM_ERROR_SIGNAL_INIT;
    else
        return COMM_OK;
}

comm_error LowLevelComm::unblockSignals()
{

    TRACE("LowLevelComm:   unblocking all signals");

    if ( sigprocmask(SIG_UNBLOCK, &alrmask, NULL) ||
            sigprocmask(SIG_UNBLOCK, &iomask, NULL)  )
        return COMM_ERROR_SIGNAL_INIT;
    else
        return COMM_OK;
}

// this simple init opens a socket to a a server with the specified
//	port and hostname
comm_error LowLevelComm::init(const char* hostname, int port)
{
    comm_error e;

    e = blockSignals();
    if(e)
        return e;

    e = createSocket (hostname, port);
    if (e)
        return e;

    e = initSigHandlers();
    if (e)
        return e;

    e = setAlarmInterval(SIMULATOR_STEP);
    if (e)
        return e;

    e = unblockSignals();
    if(e)
        return e;

    serverOk = true;


    return COMM_OK;
}

bool LowLevelComm::isServerAlive()
{
    return serverOk;
}

// handleSignals() calls finishAlarm when an alarm signal was recently received
//	and finishIo() when an io signal was recently received.
void LowLevelComm::handleSignals(SensorDataStore &_store)
{
// TODO: remove this USEPAUSE junk or optimize or somethin'
#ifdef USE_PAUSE

    TRACE(cout << "Comm: pausing" << endl);
    pause();

#else

		// TODO: figure out the difference between
		// this and blockSignals and optimize
    sigset_t oldmask;
    sigset_t blockmask;

    sigemptyset(&blockmask);
    sigaddset(&blockmask, SIGIO);
    sigaddset(&blockmask, SIGALRM);

    TRACE(cout << "Comm: blocking signals" << endl);
    sigprocmask(SIG_BLOCK, &blockmask, &oldmask); // block all

    TRACE(cout << "Comm: suspending execution" << endl);
    sigsuspend(&oldmask);

#endif

    if (alarmFlag)
    {
        alarmFlag = 0;
        finishAlarm();
    }

    if (ioFlag)
    {
        ioFlag = 0;
        finishIo(_store);
    }

#ifdef USE_PAUSE
#else
    TRACE(cout << "Comm: unblocking signals" << endl);
    sigprocmask(SIG_UNBLOCK, &blockmask, NULL);
#endif

}
