/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// playercomm.cpp Thu Jun 14 2001
// Description:
//////////////////////////////////////////////////////////////////////


#include "playercomm.h"
#include "identitydata.h"
// #include "coordinates.h"
#include "angle.h"
#include "vector2d.h"
#include "types.h"
#include "constants.h"

PlayerComm::PlayerComm()
{
}

PlayerComm::~PlayerComm()
{
}

void PlayerComm::noAction()
{
    TRACE("PlayerComm: agent doing no action");
}

void PlayerComm::sendKick(double power, double dir_rad)
{
		double dir_deg = RAD_TO_DEG( dir_rad );

    if (power < MINPOWER - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (kick " << power << " "
              << dir_deg << "), normalizing to " << MINPOWER );
        power = MINPOWER;
    }
    else if (power > MAXPOWER + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (kick " << power << " "
              << dir_deg << "), normalizing to " << MAXPOWER );
        power = MAXPOWER;
    }
    else if (dir_deg < MINMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (kick " << power << " "
              << dir_deg << "), normalizing to " << MINMOMENT );
        dir_deg = MINMOMENT;
    }
    else if (dir_deg > MAXMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (kick " << power << " "
              << dir_deg << "), normalizing to " << MAXMOMENT );
        dir_deg = MAXMOMENT;
    }
    else if (! ( (power >= MINPOWER - EPS) &&
                 (power <= MAXPOWER + EPS) &&
                 (dir_deg >= MINMOMENT - EPS) &&
                 (dir_deg <= MAXMOMENT + EPS) ) )
    {
        // NaN somewhere ?
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (kick " << power << " "
              << dir_deg << "), dropping command completely" );
        return;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(kick %f %f)", power, dir_deg);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }
}

void
PlayerComm::sendDash    (double power)
{

    if (power < MINPOWER - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (dash " << power
              << "), normalizing to " << MINPOWER );
        power = MINPOWER;
    }
    else if (power > MAXPOWER + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (dash " << power
              << "), normalizing to " << MAXPOWER );
        power = MAXPOWER;
    }
    else if (! ( (power >= MINPOWER - EPS) &&
                 (power <= MAXPOWER + EPS) ) )
    {
        // NaN ?
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (dash " << power
              << "), dropping command completely" );
        return;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(dash %f)", power);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }
}

void
PlayerComm::sendTurn    (double moment_rad)
{

    double moment_deg = RAD_TO_DEG( moment_rad );

    if (moment_deg < MINMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), normalizing to " << MINMOMENT );
        moment_deg = MINMOMENT;
    }
    else if (moment_deg > MAXMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), normalizing to " << MINMOMENT );
        moment_deg = MAXMOMENT;
    }
    else if ( ! ( (moment_deg >= MINMOMENT - EPS) &&
                  (moment_deg <= MAXMOMENT + EPS) ) )
    {
        // NaN ?
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), dropping command completely" );
        return;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(turn %f)", moment_deg);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

}

void
PlayerComm::sendCatch   (double dir_rad)
{

    double dir_deg = RAD_TO_DEG( dir_rad );

    if (dir_deg < MINMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), normalizing to " << MINMOMENT );
        dir_deg = MINMOMENT;
    }
    else if (dir_deg > MAXMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), normalizing to " << MAXMOMENT );
        dir_deg = MAXMOMENT;
    }
    else if (! ( (dir_deg >= MINMOMENT - EPS) &&
                 (dir_deg <= MAXMOMENT + EPS) ) )
    {
        // NaN ?
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), dropping command completely" );
        return;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(catch %f)", dir_deg);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

}

void
PlayerComm::sendMove    (double x, double y)
{

		// DOH: need to decide what to do here
/*    if ( (!info->isGoalie() && info->isTimeRunning() ) ||
            (info->isGoalie() && info->isTimeRunning() &&
             (info->getPlayMode() != PLAY_MODE_FREE_KICK_US) ) )
    {
        TRACE("servertime_here"
              << " PlayerComm game is running, agent not allowed to (move)" );
        return;
    }
    else
*/    {

        // no range checking when moving: there are no limits !

        char msg [MAX_SERVER_MSG_SIZE];
        sprintf(msg, "(move %f %f)", x, y);
        if (sendToServer(msg) == COMM_OK)
        {
            TRACE("PlayerComm: agent sent " << msg);
        }
        else
        {
            TRACE("servertime_here"
                  << " PlayerComm: agent cannot send " << msg << " to server" );
        }
    }

}

void
PlayerComm::sendTurnNeck   (double angle_rad)
{

    double angle_deg = RAD_TO_DEG( angle_rad );

    if (angle_deg < MINNECKMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), normalizing to " << MINNECKMOMENT );
        angle_deg = MINNECKMOMENT;
    }
    else if (angle_deg > MAXNECKMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), normalizing to " << MAXNECKMOMENT );
        angle_deg = MAXNECKMOMENT;
    }
    else if (! ( (angle_deg >= MINNECKMOMENT - EPS) &&
                 (angle_deg <= MAXNECKMOMENT + EPS) ) )
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), dropping command completely" );
        return;
    }

    // to make sure there are no (see) infos left on the socket, just read
    // and parse all outstanding IO into the SensorData buffer:
    TRACE("PlayerComm: reading all IO just before turning neck" << endl);
    // NOTE: removing finishIo() call here, b/c does not seem necessary
		// info will be picked up on next sweep
		//finishIo();

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(turn_neck %f)", angle_deg);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg << endl);

        // no need to check for twisted neck, worldmodel can handle it anyway.

          // info->twistNeck();



       // server handles turn_neck synchronously, no need to send sense_body !

           // if (see) info is received with a twisted neck an warning message
           // will be printed. Only after a sense_body arrived with a confirmed
           // head_angle will these warnings be removed.
           // ( see AsyncInfo::adjust() and AsyncInfo::isNeckTwisted() )

           //sendSenseBody();



    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

}

void
PlayerComm::sendChangeView (view_width width, view_quality quality)
{

    char *w = "";
    switch (width)
    {
    case VIEW_WIDTH_NARROW:
        w = "narrow";
        break;
    case VIEW_WIDTH_NORMAL:
        w = "normal";
        break;
    case VIEW_WIDTH_WIDE:
        w = "wide";
        break;
    }

    char *q = "";
    switch (quality)
    {
    case VIEW_QUALITY_LOW:
        q = "low";
        break;
    case VIEW_QUALITY_HIGH:
        q = "high";
        break;
    }

    char msg[MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(change_view %s %s)", w, q);
    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        TRACE("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

}

