/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// turncommand.cpp Fri Jun 22 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "turncommand.h"

#undef TRACE
#define TRACE(cmd)

TurnCommand::TurnCommand() : ServerCommand(1)
{
}

TurnCommand::TurnCommand(double _dMomentRad) : ServerCommand(1)
{
	setTurnMoment(_dMomentRad);
}

TurnCommand::~TurnCommand()
{
}

Comm& TurnCommand::comm_insert(Comm &_cmConnection) const
{
    double moment_deg = RAD_TO_DEG( dMomentRad );

    if (moment_deg < MINMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), normalizing to " << MINMOMENT );
        moment_deg = MINMOMENT;
    }
    else if (moment_deg > MAXMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), normalizing to " << MINMOMENT );
        moment_deg = MAXMOMENT;
    }
    else if ( ! ( (moment_deg >= MINMOMENT - EPS) &&
                  (moment_deg <= MAXMOMENT + EPS) ) )
    {
        // NaN ?
        TRACE("servertime_here" << " PlayerComm: agent tries to send (turn " << moment_deg
              << "), dropping command completely" );
        return _cmConnection;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(turn %f)", moment_deg);
    if (_cmConnection.sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        ALERT("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

	return _cmConnection;
}

void TurnCommand::setTurnMoment(double _dMomentRad)
{
	dMomentRad = _dMomentRad;
}

double TurnCommand::getTurnMoment() const
{
	return dMomentRad;
}
