/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// debug.h Fri Jun 8 2001
// Description: This file contains all the debug macros to be used
//							throughout the project.
//////////////////////////////////////////////////////////////////////


#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <iostream.h>
#include <assert.h>

// Necessary in replayer, until we figure out a fix for this
/*
class Recommendation {
public:
	string szBehaviorName;
	double grade;
};
*/

//////////////////////////////////////////////////////////////////////
// Usage: In general, use TRACE when you want to send non-critical,
//				but noteworthy information to the console. TRACE commands
//				are ment to be permanently in the code.  Use ALERT
//				to send critical messages to the console.  ALERT commands
//				are meant to be permanently in the code.  Use INSPECT
//				to print out the value of variables or indicate when
//				a datastructure or function has been entered.  INSPECT
//				is meant to be temporary and only for debugging specific
//				sections of code.
//////////////////////////////////////////////////////////////////////


#define COMPETITION

#ifdef COMPETITION
#define TRACE(message)
#else
#define TRACE(message) cout << __FILE__ << ":" << __LINE__ << ": " << message << endl;
#endif

#ifdef COMPETITION
#define ALERT(message)
#else
#define ALERT(message) cerr << __FILE__ << ":" << __LINE__ << ": " << message << endl;
#endif

#ifdef COMPETITION
#define ENSURE(expr)					\
{							\
  if (!(expr)) {					\
    cerr << "Warning: ensure " << #expr << " not valid in "	\
	 << __FILE__ << ":" << __LINE__ << endl;	\
  }							\
}
#else
#define ENSURE(expr) assert(expr)
#endif


#ifdef COMPETITION
#define INSPECT(message)
#else
#define INSPECT(message) cerr << __FILE__ << ":" << __LINE__ << ": " << message << endl;
#endif

#endif

