/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// actionlist.cpp Fri Jul 20 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "actionlist.h"
#include "debug.h"

ActionList::ActionList()
{
}

ActionList::ActionList(const ActionList& alOther)
{
	clear();
	list<Action*>::const_iterator i;
	
	for(i = alOther.lActionPtrs.begin(); i != alOther.lActionPtrs.end(); ++i)
	{
		lActionPtrs.push_back( (*i)->copy() );
	}
}

ActionList::~ActionList()
{
	while(!lActionPtrs.empty())
	{
		Action* pacAction;
		pacAction = lActionPtrs.back();
		delete pacAction;
		lActionPtrs.pop_back();
	}
}

void ActionList::clear()
{
	while(!lActionPtrs.empty())
	{
		delete lActionPtrs.back();
		lActionPtrs.pop_back();
	}
}


/*
void ActionList::push_back(Action* _pacAction)
{	
	lActionPtrs.push_back(_pacAction->copy());
}
*/

void ActionList::insert(const Action& _acAction)
{
	Action* pacNew = _acAction.copy();
	
	if (pacNew->getDuration() == 0)
		lActionPtrs.push_front(pacNew);
    else	
	    lActionPtrs.push_back(pacNew);
}


unsigned int ActionList::size() const
{
	return lActionPtrs.size();
}

/*
Action* ActionList::operator[](unsigned int i) const
{
	ENSURE ( (i < lActionPtrs.size()) && (i >= 0) );		
		
	return lActionPtrs[i];
}
*/

Action* ActionList::front() const
{
	return lActionPtrs.front();
}

void ActionList::pop_front()
{
	lActionPtrs.pop_front();
}

bool ActionList::empty() const
{
	return lActionPtrs.empty();
}

// deprecated

/*
Action& ActionList::operator[](unsigned int i) const
{
	ENSURE((i < lActionPtrs.size()) && (i >= 0));

	return *lActionPtrs[i];
}
*/

/*
ActionList ActionList::merge(const ActionList& alOther) const
{
	ActionList alNew = *this;
	copy(alOther.lActionPtrs.begin(), alOther.lActionPtrs.end(), back_inserter(alNew.lActionPtrs) );
	
	return alNew;
}
*/

ActionList& ActionList::operator=(const ActionList &alOther)
{
	if ( &alOther == this )
		return *this;
	
	clear();
	
	list<Action*>::const_iterator i;	
	
	for(i = alOther.lActionPtrs.begin(); i != alOther.lActionPtrs.end(); ++i)
	{
		lActionPtrs.push_back( (*i)->copy() );
	}
	
	return *this;
}

ofstream& ActionList::ofstream_insert(ofstream& _fout) const
{
    return _fout;
}

ostream& ActionList::ostream_insert(ostream& _out) const
{
	list<Action*>::const_iterator i;
	
    for(i = lActionPtrs.begin(); i != lActionPtrs.end(); ++i)
    {
        if(*i) _out << *(*i);
    }

    return _out;
}

ofstream& operator<<(ofstream& _fout, const ActionList& _alList)
{
    return _alList.ofstream_insert(_fout);
}

ostream& operator<<(ostream& _out, const ActionList& _alList)
{
    return _alList.ostream_insert(_out);
}
