/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// stayonfieldperception.cpp Thu Jul 12 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "stayonfieldperception.h"
#include "minmax.h"

StayOnFieldPerception::StayOnFieldPerception(const WorldModel& _wmWorld) : Perception(_wmWorld)
{
}

StayOnFieldPerception::~StayOnFieldPerception()
{
}

bool StayOnFieldPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_PLAY_ON) ||
        (pmTemp == PLAY_MODE_DROP_BALL)
      ) return true;

    return false;
}

bool StayOnFieldPerception::amIOnField() const
{
    Vector2d vPos = wmWorld.getClientPlayer().getAbsPosition();

    if( (fabs(vPos.getX()) > FIELD_LENGTH / 2) ||
        (fabs(vPos.getY()) > FIELD_WIDTH / 2) ) return false;

    return true;
}

bool StayOnFieldPerception::amICloseToSide() const
{
    Vector2d vPos = wmWorld.getClientPlayer().getAbsPosition();

    if( (fabs(vPos.getX()) > FIELD_LENGTH / 2 - CLOSE_MARGIN) ||
        (fabs(vPos.getY()) > FIELD_WIDTH / 2 - CLOSE_MARGIN) ) return true;

    return false;
}

Grade StayOnFieldPerception::computeGrade() const
{
        Vector2d vPos = wmWorld.getClientPlayer().getAbsPosition();

        // Distance to side
        float distX = FIELD_LENGTH / 2 - fabs(vPos.getX());
        float distY = FIELD_WIDTH / 2 - fabs(vPos.getY());
        if(distX < 0) distX = 0;
        if(distY < 0) distY = 0;

        double dGrade = (CLOSE_MARGIN - min(distX,distY))/CLOSE_MARGIN * 99;
				
				if (dGrade < 0.0)
					dGrade = 0.0;

        return dGrade;
}

Angle StayOnFieldPerception::getHomeDir() const
{
	Vector2d vRunAngle = relPos( Vector2d(0.0, 0.0) );
	return (vRunAngle.getAngle());
}
