/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// worldmodel.cpp Fri Jun 8 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "worldmodel.h"
#include "polygon.h"

#undef TRACE
#define TRACE(cmd)

WorldModel::WorldModel() : BaseWorldModel()
{
	pwpClientPlayer = NULL;
}

WorldModel::~WorldModel()
{
}

void WorldModel::updateWorldBall(const SensorObject &_soBall)
{

	// set the cone time to zero and increase the age	
	if (wbBall.getAge() == 0) wbBall.setConeTime(0);
 	
   // and, assuming our sensory information is valid
	if (_soBall.valid)
	{		
			// and update the ball's absolute position
	    Vector2d vRelPos;
        Angle aFaceDir = pwpClientPlayer->getFaceDir();
     	vRelPos.setRadAng(_soBall.dist, _soBall.dir + aFaceDir);
	    wbBall.setAbsPosition(vRelPos + getOrigin());
			
		// and update the ball's absolute velocity
		Vector2d vVelocity = _soBall.dist_change * vRelPos / vRelPos.getLength();	
		vRelPos.perp();
		vVelocity += _soBall.dir_change * vRelPos + pwpClientPlayer->getVelocity();
		
		if (vVelocity.getLength() <= wbBall.getMaxVelocity())
		{
				wbBall.setVelocity(vVelocity);
		    //vVelocity.setRadAng(wbBall.getMaxVelocity(), vVelocity.getAngle());
		}
		

		
			
/*		Vector2d vRelVelocity;
		vRelVelocity.setRadAng(_soBall.dist_change, _soBall.dir + aFaceDir);
		Vector2d vAbsVelocity = pwpClientPlayer->getVelocity() + vRelVelocity;
	
		if(vAbsVelocity.getLength() > BALL_SPEED_MAX)
		{
		    TRACE("Error: Ball speed exceeding maximum, attempted value: " << vAbsVelocity.getLength());
		    wbBall.setVelocity(BALL_SPEED_MAX, vAbsVelocity.getAngle());
		}
		else
		{
			    // WARNING: test
			    if (vAbsVelocity.getX() < 0)
    			{
    				double x = vAbsVelocity.getY();
    				double y = vAbsVelocity.getX();
    				vAbsVelocity.setXY(-x,-y);
    			} // end Warning
    						
			
			    wbBall.setVelocity(vAbsVelocity);
			}
*/			
			
    	wbBall.increaseConeTime();	
    	wbBall.setAge(0);
	}
}

void WorldModel::updateWorldBall(const WorldBall& _wbBall)
{
	if(_wbBall.getAge() > wbBall.getAge())
		return;

    wbBall.setAbsPosition( _wbBall.getAbsPosition() );
    wbBall.setVelocity(_wbBall.getVelocity() );
    wbBall.setAge( _wbBall.getAge() );
}



void WorldModel::updatePlayer(const SensorPlayer &_spPlayer)
{
		// TODO: implement update functions with worldplayers & multiple
		//			 inheritence (WorldPlayer/have-a clientplayer objects)
    WorldPlayer *pwpThisPlayer = NULL;
    if(!_spPlayer.valid) return;
	if (!((_spPlayer.number >= 1) && (_spPlayer.number <= MAX_PLAYER)))
	{
	   TRACE( "Error!  Valid player with invalid number received!!!" << endl );
	   return;
	}
	
    if (_spPlayer.team == TEAM_US) {
        pwpThisPlayer = pwpOurTeam[_spPlayer.number - 1];
        if(_spPlayer.goalie == GOALIE_TRUE) setOurGoalie(pwpThisPlayer);
    }
    else if (_spPlayer.team == TEAM_THEM) {
        pwpThisPlayer = pwpOtherTeam[_spPlayer.number - 1];
        if(_spPlayer.goalie == GOALIE_TRUE) setOtherGoalie(pwpThisPlayer);
    }
    else
    {
        TRACE( "Error! Valid player with valid number and invalid team received!" << endl );
        return;
    }

    // if this is the second time we've seen this guy increment his cone time
    if(pwpThisPlayer->getAge() == 0) pwpThisPlayer->increaseConeTime();

    // reset age to zero and set known
    pwpThisPlayer->setAge(0);
    pwpThisPlayer->setKnown();

    // set the absolute position of the player
		Vector2d vRelPos;
		Angle aFaceDir = pwpClientPlayer->getFaceDir();
		vRelPos.setRadAng(_spPlayer.dist, _spPlayer.dir + aFaceDir);
		
		// Never get yourself as a visual artifact, getOrigin() operates on ClientPlayer (self)
		pwpThisPlayer->setAbsPosition(getOrigin() + vRelPos);

		// and update the player's absolute velocity
		Vector2d vVelocity = _spPlayer.dist_change * vRelPos / vRelPos.getLength();	
		vRelPos.perp();
		vVelocity += _spPlayer.dir_change * vRelPos + pwpClientPlayer->getVelocity();
		
		if (vVelocity.getLength() <= pwpThisPlayer->getMaxVelocity())
		{
				pwpThisPlayer->setVelocity(vVelocity);
		}

	
		// set body and face direction
		pwpThisPlayer->setBodyDir(_spPlayer.body_dir);
		pwpThisPlayer->setHeadDir(_spPlayer.head_dir);
		
		// set the players jersey number
		pwpThisPlayer->setNumber(_spPlayer.number);
		
		// is the player the goalie?
		if (_spPlayer.goalie == GOALIE_TRUE)
			pwpThisPlayer->setGoalie(true);
		else
			pwpThisPlayer->setGoalie(false);

//    if (_spPlayer.team == TEAM_US)
//    {
//        if(pwpThisPlayer->isGoalie()) setOurGoalie(pwpThisPlayer);
//    }
//    else if (_spPlayer.team == TEAM_THEM) {
//        pwpThisPlayer = pwpOtherTeam[_spPlayer.number - 1];
//        if(pwpThisPlayer->isGoalie()) setOtherGoalie(pwpThisPlayer);
//    }

}

void WorldModel::setClientPlayerNum(int _iClientPlayerNum)
{
	// replace the player with the jersey number _iClientPlayerNum
	// with a new client player
	ENSURE((_iClientPlayerNum >= 1) && (_iClientPlayerNum <= MAX_PLAYER));
	
	delete pwpOurTeam[_iClientPlayerNum - 1];
	
	pwpClientPlayer = new ClientWorldPlayer();
	pwpOurTeam[_iClientPlayerNum - 1] = pwpClientPlayer;
	
	// TODO: Make the unnecessary (modify constructor to force it) JM
	pwpClientPlayer->setTeam(TEAM_US);
	pwpClientPlayer->setNumber(_iClientPlayerNum);
}

ClientWorldPlayer WorldModel::getClientPlayer() const
{
	ENSURE(pwpClientPlayer != NULL);
	return *pwpClientPlayer;
}

ClientWorldPlayer* WorldModel::modifyClientPlayer()
{
	return pwpClientPlayer;
}


Vector2d WorldModel::getOrigin() const
{
	return pwpClientPlayer->getAbsPosition();
}

void WorldModel::computeOrigin(const SensorFlag _sfFlags[], const SensorLine _slLines[])
{
    // TODO: comment this function and clean it up
    int   numLinesFound = 0;   // number of lines player has seen
    int   numFlagsFound = 0;   // number of flags player has seen

    // step 1 : count valid flags and save the two closest ones

    Angle aFaceDir;						 // the face directon of the client player from flags
    int closest = UNKNOWN;
    double closestDist = INFINITY;
    int second = UNKNOWN;
    double secondDist = INFINITY;

    for( int i = 0; i < NUM_FLAGS; i++ )
    {
        if( _sfFlags[i].valid && _sfFlags[i].which != FLAG_UNKNOWN
                && IsDistValid( _sfFlags[i].dist && IsDirValid( _sfFlags[i].dir )) )
        {
            numFlagsFound++;
            if( _sfFlags[i].dist < secondDist )
            {
                second = i;
                secondDist = _sfFlags[i].dist;
            }
            if( _sfFlags[i].dist < closestDist )
            {
                second = closest;
                secondDist = closestDist;
                closest = i;
                closestDist = _sfFlags[i].dist;
            }
        }
    }

    if( numFlagsFound == 0 )
    {
        TRACE("Cannot determine any flag!");
    }

    // step 2 : try to calculate face direction with one line

    for( int i = 0; i < NUM_LINES; i++ )
    {
        if( _slLines[i].valid && _slLines[i].which != LINE_UNKNOWN )
        {
            numLinesFound++;

            if( numLinesFound > 1 )   // this means player is outside the field
						{
							ALERT("Player is outside the field!");
						}

            Angle alpha = _slLines[i].dir;

            switch( _slLines[i].which )
            {
            case LINE_RIGHT:
                alpha = alpha > 0 ? PI_2 - alpha : -PI_2 - alpha;
                break;

            case LINE_BOTTOM:
                alpha = alpha > 0 ? PI - alpha : -alpha;
                break;

            case LINE_LEFT:
                alpha = alpha > 0 ? -PI_2 - alpha : PI_2 - alpha;
                break;

            case LINE_TOP:
                alpha = alpha > 0 ? -alpha : -PI - alpha;
                break;

            default:
                ENSURE( false );
            }

            Angle l( leftDirLimit( alpha ) );
            Angle r( rightDirLimit( alpha ) );
            alpha = bisect( l, r );
            aFaceDir = alpha;
        }
    }
		
    // step 3 : calculate face direction, if it could not be calculated with one line

    if( numLinesFound != 1 )
    {
        if( numFlagsFound == 1 )
        {
            TRACE( "Only determined one flag!" << endl );
        }

        Vector2d diffAbsPos = FLAGPOS( _sfFlags[closest].which ) - FLAGPOS( _sfFlags[second].which );

        Vector2d diffRelPos = radAng2Vector2d( _sfFlags[closest].dist, _sfFlags[closest].dir )
                              - radAng2Vector2d( _sfFlags[second].dist, _sfFlags[second].dir );

        aFaceDir = diffAbsPos.getAngle() - diffRelPos.getAngle();
    }

    // step 4 : calculate absolute position with polygon intersection
    //          if intersection fails use old method (weighted use of flag polygons)

    Polygon posPoly;                      // intersection polygon of all flag polygons
    bool firstFlag = true;                // is the current flag the first valid flag?

    Vector2d myWeightedPos( 0.0, 0.0 );   // my position calculated with old method (if intersecting fails)
    double weightsum = 0.0;               // weightsum for old calculation method

    for( int i = 0; i < NUM_FLAGS; i++ )
    {
        if( _sfFlags[i].valid && _sfFlags[i].which != FLAG_UNKNOWN
                && IsDistValid( _sfFlags[i].dist ) )
        {
            Polygon flagPoly;   // position polygon for current flag

            Vector2d flagpos = FLAGPOS( _sfFlags[i].which );   // get flag position

            // de-quantized absolute area of flag position
            double top = upperDistLimit( _sfFlags[i].dist, QUANTIZE_STEP_L );
            double bot = lowerDistLimit( _sfFlags[i].dist, QUANTIZE_STEP_L );

            // assume maximal error of 1.5 deg
            Angle l( leftDirLimit( _sfFlags[i].dir ) + aFaceDir - DEG_TO_RAD( 1.5 ) );
            Angle r( rightDirLimit( _sfFlags[i].dir ) + aFaceDir + DEG_TO_RAD( 1.5 ) );

            // limitations of our own position, as determined by flag
            Vector2d lowleft = flagpos - radAng2Vector2d( top, r );
            Vector2d lowmiddle = flagpos - radAng2Vector2d( top, bisect( l, r ) );
            Vector2d lowright = flagpos - radAng2Vector2d( top, l );
            Vector2d hileft = flagpos - radAng2Vector2d( bot, r );
            Vector2d hiright = flagpos - radAng2Vector2d( bot, l );

            if( firstFlag )
            {
                // just fill posPoly (nothing to intersect yet)
                posPoly.addPoint( lowleft );
                posPoly.addPoint( hileft );
                posPoly.addPoint( hiright );
                posPoly.addPoint( lowright );
                posPoly.addPoint( lowmiddle );

                firstFlag = false;
            }
            else
            {
                // intersect posPoly with new flag polygon
                flagPoly.addPoint( lowleft );
                flagPoly.addPoint( hileft );
                flagPoly.addPoint( hiright );
                flagPoly.addPoint( lowright );
                flagPoly.addPoint( lowmiddle );

                posPoly ^= flagPoly;
            }

            // old position calculation method
            double weight = 1.0 / ( 1.0 + _sfFlags[i].dist );
            weightsum += weight;
            myWeightedPos += ( flagpos - radAng2Vector2d( 0.5 * ( top + bot ), bisect( l, r ) ) ) * weight;
        }
    }

    Vector2d vAbsPosition;

    if( posPoly.nop() > 0 )
    {
        // new method (polygon intersection) worked
        // pwpClientPlayer->setAbsPosition(posPoly.getInside());
        vAbsPosition = posPoly.getInside();
    }
    else if (myWeightedPos != Vector2d(0.0, 0.0))
    {
        // new method (polygon intersection) failed, use weighted position
        // pwpClientPlayer->setAbsPosition(myWeightedPos * ( 1.0 / weightsum ));
        vAbsPosition = myWeightedPos / weightsum;
    }
		// else keep information from last round
		
    // The server passes information relative to the client player.  When the client player is on the right side
    // of the field, the absolute position of worldobjects are flipped so that, as far as our functions are assumed
    // the client player is playing from SIDE_LEFT

  	if( pwpClientPlayer->getPlayingSide() == SIDE_RIGHT )
    {
      vAbsPosition.perp();
      vAbsPosition.perp();
      aFaceDir += PI;
    }

  	pwpClientPlayer->setAbsPosition(vAbsPosition);
    pwpClientPlayer->setFaceDir(aFaceDir);

    TRACE("AbsolutePosition: " << vAbsPosition.getX() << ", " << vAbsPosition.getY());
}

void WorldModel::age()
{
    BaseWorldModel::age();

    // I didn't test this pointer for efficiency
    // logic: if we don't know who the client player is we want it to crash
    pwpClientPlayer->setAge(0);
}

/*
void WorldModel::setTime(int _iTime)
{
    if(_iTime != iTime) // if the time has changed
        age(); // age the worldmodel

    iTime = _iTime;
}
*/

// Read binary data from log file
ifstream& operator>>(ifstream& _in, WorldModel &_wmWorld)
{
  return _wmWorld.ifstream_extract(_in);
}

ofstream& operator<<(ofstream& _out, WorldModel &_wmWorld)
{
  return _wmWorld.ofstream_insert(_out);
}


ofstream& WorldModel::ofstream_insert(ofstream& _out)
{
    _out << "<baseworldmodel time=\"" << getTime() << "\"" << " playmode=\"" << getPlayMode() << "\">" << endl;

    // TODO (optional): Make a ball special, and not just a worldmovable object
    _out << "  <ball>" << endl;
    getWorldBall().ofstream_insert(_out);
    _out << "  </ball>" << endl;

    // Log all active players
    for(int i=0; i < MAX_PLAYER; ++i) {
        if(pwpOurTeam[i] && (pwpOurTeam[i]->getAge() != UNKNOWN)) {
            pwpOurTeam[i]->ofstream_insert(_out);
        }
    }
    for(int i=0; i < MAX_PLAYER; ++i) {
        if(pwpOtherTeam[i] && (pwpOtherTeam[i]->getAge() != UNKNOWN)) {
            pwpOtherTeam[i]->ofstream_insert(_out);
        }
    }

    // Log goals
    _out << "  <ourgoal absx=\"" << getOurGoal().getAbsPosition().getX()
         << "\" absy=\"" << getOurGoal().getAbsPosition().getY() << "\"/>" << endl;

    _out << "  <othergoal absx=\"" << getOtherGoal().getAbsPosition().getX()
         << "\" absy=\"" << getOtherGoal().getAbsPosition().getY() << "\"/>" << endl;

    _out << "</baseworldmodel>" << endl;

    return _out;
}


int WorldModel::getVisualTimeStamp() const
{
	return iLastVisualTimeStamp;
}

void WorldModel::setVisualTimeStamp(int _iLastVisualTimeStamp)
{
	iLastVisualTimeStamp = _iLastVisualTimeStamp;
}
