/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// worldplayer.h Fri Jun 8 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef _WORLDPLAYER_H_
#define _WORLDPLAYER_H_

#include <iostream.h>
#include <fstream.h>

#include "worldmovableobject.h"
#include "types.h"
#include "constants.h"
#include "sensor.h"

// TODO: move methods that pertain only to the client player and
//			 not to all players into the client player class
class WorldPlayer : public WorldMovableObject
{
public: 
	WorldPlayer();
	WorldPlayer(const Vector2d &_vAbsPosition);
	WorldPlayer(int _iNumber, team_number _tnTeam, bool _fGoalie);
	virtual ~WorldPlayer();

	//inspectors
	virtual Angle getBodyDir () const;
	virtual Angle getHeadDir () const;
	int getNumber() const;
	bool isGoalie() const;
	team_number getTeam() const;
	double getStamina() const;
  double getEffort() const;

	//computes the direction the player is facing
	virtual Angle getFaceDir () const;
	
	//mutators
	void setBodyDir (Angle _aBodyDir);
  void setHeadDir (Angle _aHeadDir);
	void setTeam (team_number _tnTeam);
	void setStamina(const double &_dStamina);
  void setEffort(const double &_dEffort);
	void setNumber (int _iNumber);
	void setGoalie (bool _fGoalie); 	
	bool isYoung(int _iAge = 6);

//    WorldPlayer& operator+=(const SensorPlayer &spPlayer);
//    WorldPlayer& operator+=(const ExactPlayer &_ePlayer); // coach facility


    virtual ofstream& ofstream_insert(ofstream& _fout);
    virtual ostream& ostream_insert(ostream& _out);
    virtual ifstream& ifstream_extract(ifstream& _in);

    bool operator == ( const WorldPlayer& _wpPlayer ) const;


private:

	Angle  	aBodyDir;   // player's body-and-running direction
    Angle 	aHeadDir;   // player's head angle in relation to body

    int  		iNumber;    // player's number ( -1 = undefined, 0 = coach )
    bool 		fGoalie;
	
    // which team is this player on? don't use much since two arrays, one for each team
 	team_number tnTeam;
	
	// effort and stamina for each player, for the moment, this is unknown for most players
	double dStamina;
	double dEffort;

	// what side of the field is this player, playing on...
};


ostream& operator<<(ostream& _out, WorldPlayer &_wpPlayer);

#endif
